;;; -----------------------------------------------------------------------------
;;; Plugin  : Lizard-Breath_RD.scm (for lack of a better name)
;;; Author  : Rod Detmer
;;; Date    : August 15th 2012				
;;; 
;;; Creates a reptile text effect on a user created selected text layer
;;; -----------------------------------------------------------------------------
;;; Change Log: Version-2.0
;;;             Graechan added shine to filter
;;; Change Log: Version-3.0
;;;             MareroQ added: text, direction, justify, letter-spacing, line-spacing, border, Color Text Edge, Thickness Text Edge, Color Inside the Text, displace-type, Alpha to Logo + Active Layer to New Image.
;;; Information can be found at http://www.gimpchat.com
;;; -----------------------------------------------------------------------------
; License: GPLv3
; This program is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; To view a copy of the GNU General Public License
; visit: http://www.gnu.org/licenses/gpl.html
;;; -----------------------------------------------------------------------------

(define (resize-image-to-layers-by-border-lb inImg inDrawable inBufferSize)
    (let* (
        (theImgWidth (car(gimp-image-width inImg)))
        (theImgHeight (car(gimp-image-height inImg)))
        (theIterations 0)
        (theLayerNum (car(gimp-image-get-layers inImg)))
        (theWorkingLayer 0)
        (theLayerWidth 0)
        (theLayerHeight 0)
        (theLayerOffsetX 0)
        (theLayerOffsetY 0)
        (theBorder FALSE)
        (theMinPosX theImgWidth)
        (theMaxPosX 0)
        (theMinPosY theImgHeight)
        (theMaxPosY 0)
        (theNewImgWidth theImgWidth)
        (theNewImgHeight theImgHeight)
        )

    (gimp-image-undo-group-start inImg)

    (begin
     (while (< theIterations theLayerNum)
        (set! theWorkingLayer (aref (cadr (gimp-image-get-layers inImg)) theIterations))
        (gimp-image-set-active-layer inImg theWorkingLayer)

        (set! theLayerOffsetX (car(gimp-drawable-offsets theWorkingLayer)))
        (set! theLayerOffsetY (cadr(gimp-drawable-offsets theWorkingLayer)))
        (set! theLayerWidth (car(gimp-drawable-width theWorkingLayer)))
        (set! theLayerHeight (car(gimp-drawable-height theWorkingLayer)))

        (cond ((> (+ theLayerWidth theLayerOffsetX) theImgWidth) (set! theBorder TRUE))
              ((> (+ theLayerWidth theLayerOffsetX) theMaxPosX) (set! theMaxPosX (+ theLayerWidth theLayerOffsetX)))
        );end cond

        (cond ((> (+ theLayerHeight theLayerOffsetY) theImgHeight) (set! theBorder TRUE))
              ((> (+ theLayerHeight theLayerOffsetY) theMaxPosY) (set! theMaxPosY (+ theLayerHeight theLayerOffsetY)))
        );end cond

        (cond ((< theLayerOffsetX 0) (set! theBorder TRUE))
              ((< theLayerOffsetX theMinPosX) (set! theMinPosX theLayerOffsetX))
        );end cond

        (cond ((< theLayerOffsetY 0) (set! theBorder TRUE))
              ((< theLayerOffsetY theMinPosY) (set! theMinPosY theLayerOffsetY))
        );end cond

        (set! theIterations (+ theIterations 1))
     );end while
    );end begin

    (if (equal? theBorder TRUE) (set! theMinPosX 0) (set! theMaxPosX (- theMaxPosX theMinPosX)))
    (if (equal? theBorder TRUE) (set! theMinPosY 0) (set! theMaxPosY (- theMaxPosY theMinPosY)))
    (if (equal? theBorder TRUE) (set! theMaxPosX (- theImgWidth theMinPosX)))
    (if (equal? theBorder TRUE) (set! theMaxPosY (- theImgHeight theMinPosY)))

    ;resize the image
    (gimp-image-resize inImg theMaxPosX theMaxPosY (- theMinPosX) (- theMinPosY))

    ;add the border. Default is 0, only useful if set by the user
    (set! theImgWidth (+ (car(gimp-image-width inImg)) inBufferSize inBufferSize)) ;szerokość
    (set! theImgHeight (+ (car(gimp-image-height inImg)) inBufferSize inBufferSize))			;wysokość

    ;resize the image to add the border
    (gimp-image-resize inImg theImgWidth theImgHeight inBufferSize inBufferSize)

    (gimp-selection-none inImg)
	
	(map (lambda (x) (gimp-layer-resize-to-image-size x)) (vector->list (cadr (gimp-image-get-layers inImg))))
    (gimp-image-undo-group-end inImg)
    (gimp-displays-flush)
    );end let
);end define
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (the-lizard-breath-text-shine image drawable shadow-size shadow-opacity displace-type conserve )
                      
(let* (
         (image-layer (car (gimp-image-get-active-layer image)))
         (width (car (gimp-image-width image)))
         (height (car (gimp-image-height image)))
         (sel (car (gimp-selection-is-empty image)))
         (layer-name (car (gimp-drawable-get-name image-layer)))
         (img-layer 0)
         (img-channel 0)
         (bkg-layer 0)
         (shadow-layer 0)
         (tmp-layer 0)
         (displace-type (cond ((= displace-type 0) 1)
			                  ((= displace-type 1) 2)
			                  ((= displace-type 2) 3)))
        )
      
   (gimp-context-push)
   (gimp-image-undo-group-start image)
   (gimp-context-set-foreground '(0 0 0))
   (gimp-context-set-background '(255 255 255))
   (gimp-layer-add-alpha image-layer)
   (if (= sel TRUE) (gimp-selection-layer-alpha image-layer))
   (set! img-layer (car (gimp-layer-new image width height RGBA-IMAGE "img-layer" 60 LAYER-MODE-NORMAL-LEGACY)))
   (gimp-image-insert-layer image img-layer 0 -1)
   (gimp-drawable-fill img-layer  FILL-BACKGROUND)
   (gimp-edit-fill img-layer FILL-FOREGROUND)
;;;;create channel
   (gimp-selection-save image)
   (set! img-channel (car (gimp-image-get-active-drawable image)))   
   (gimp-channel-set-opacity img-channel 100)   
   (gimp-drawable-set-name img-channel "img-channel")
   (gimp-image-set-active-layer image img-layer)   
   (gimp-drawable-set-name image-layer "Original Image")
   
;;;;create the background layer   
    (set! bkg-layer (car (gimp-layer-new image width height RGBA-IMAGE "Background" 100 LAYER-MODE-NORMAL-LEGACY)))
    (gimp-image-insert-layer image bkg-layer 0 1)
      
;;;;apply the image effects
   (gimp-context-set-foreground '(0 0 0))
   (gimp-context-set-background '(255 255 255))
   (plug-in-gauss-rle2 RUN-NONINTERACTIVE image img-layer 12 12)
   (plug-in-emboss RUN-NONINTERACTIVE image img-layer 225 84 10 TRUE)   
   (gimp-selection-invert image)
   (gimp-edit-clear img-layer)
   (gimp-selection-invert image)
   (plug-in-colortoalpha RUN-NONINTERACTIVE image img-layer '(254 254 254));;fefefe
   (plug-in-gauss-rle2 RUN-NONINTERACTIVE image img-channel 15 15)
   (plug-in-blur RUN-NONINTERACTIVE image img-layer)
   (gimp-image-set-active-layer image bkg-layer)
  ;(plug-in-displace run-mode image drawable amount-x amount-y do-x do-y displace-map-x displace-map-y displace-type)
  ; displace-type Edge behavior { WRAP (1), SMEAR (2), BLACK (3) } (1 <= displace-type <= 3)
   (plug-in-displace RUN-NONINTERACTIVE image bkg-layer 8 8 TRUE TRUE img-channel img-channel displace-type)
   (gimp-image-remove-layer image bkg-layer)
   
;;;;create the shadow
(if (> shadow-size 0)
  (begin
    (script-fu-drop-shadow image img-layer shadow-size shadow-size shadow-size '(0 0 0) shadow-opacity FALSE)
    (set! tmp-layer (car (gimp-layer-new image width height RGBA-IMAGE "temp" 100 LAYER-MODE-NORMAL-LEGACY)))
    (gimp-image-insert-layer image tmp-layer 0 -1)
    (gimp-image-raise-layer image tmp-layer)
    (gimp-image-merge-down image tmp-layer CLIP-TO-IMAGE)
    (set! shadow-layer (car (gimp-image-get-active-drawable image)))
    (gimp-image-lower-layer image shadow-layer)
   )
)   

(if (= conserve FALSE)
   (begin
   (set! img-layer (car (gimp-image-merge-down image img-layer EXPAND-AS-NECESSARY)))
   (set! img-layer (car (gimp-image-merge-down image img-layer EXPAND-AS-NECESSARY)))
   (gimp-drawable-set-name img-layer layer-name)
   )
   )   
   (gimp-image-remove-channel image img-channel)
   (gimp-selection-none image)
   (gimp-image-undo-group-end image)
   (gimp-context-pop)
   (gimp-displays-flush)
)
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (script-fu-lizard-breath-efect image layer ScaleSize color thickness colorin ScaleShape displace-type conserve)
  (gimp-image-undo-group-start image)
  (gimp-image-select-item image 0 layer)
  (gimp-context-set-foreground colorin)
  (gimp-context-set-background '(0 0 0))   
  (gimp-edit-fill layer 0)
  (gimp-layer-resize-to-image-size layer)
  (plug-in-mosaic RUN-NONINTERACTIVE image layer ScaleSize 4 2 0.6 0 120 0.2 0 0 ScaleShape 0 1) 
  (gimp-by-color-select layer colorin 15 0 1 0 0 1)
  (plug-in-emboss RUN-NONINTERACTIVE image layer 120 30 5 0)
  (gimp-selection-none image)
  (plug-in-neon RUN-NONINTERACTIVE image layer 1.75 0.25)
  (gimp-context-set-background '(255 255 255))
  (gimp-context-set-foreground '(0 0 0))
  (gimp-image-select-item image 0 layer) 
  (gimp-selection-grow image 2)
  (gimp-selection-border image thickness)
  (gimp-context-set-background '(0 0 0))
  (gimp-context-set-foreground color) 
  (gimp-edit-fill layer 0)   
  (python-layer-fx-drop-shadow 1 image layer '(0 0 0) 69 0 0 0 7 7 120 5 0 0)
  (gimp-context-set-background '(255 255 255))
  (gimp-context-set-foreground '(0 0 0)) 
  (gimp-selection-none image)
 
;;;;Shine added here
  (let* ((shadow-size 8) (shadow-opacity 50))

  (the-lizard-breath-text-shine image layer shadow-size shadow-opacity displace-type conserve))
  (gimp-image-undo-group-end image) 
  (gimp-displays-flush image) 
)

    ;register menu location and information   
(define (script-fu-lizard-breath-logo text
								   direction
								   justify
								   letter-spacing
								   line-spacing								   
                                   size
								   border
                                   font
								   ScaleSize
								   color
								   thickness
								   colorin
								   ScaleShape
								   displace-type
								   conserve)
 (let* (
          (image -1)                                 ; Declare and Init img
          (text-layer -1)                          ; Declare and init text-layer
          (logo-layer -1) 								   
        (justify (cond ((= justify 0) 2)
		               ((= justify 1) 0)
					   ((= justify 2) 1)
					   ((= justify 3) 3)))
		)			   
  (begin ;else
        (set! image (car (gimp-image-new 128 128 RGB)))   ; MrQ
        (set! text-layer (car (gimp-text-fontname image -1 0 0 text 0 TRUE size PIXELS font)))
	
;;;;adjust the text-layer ;add MrQ		

	(gimp-text-layer-set-base-direction text-layer direction)
    (gimp-text-layer-set-justification text-layer justify)
	(gimp-text-layer-set-letter-spacing text-layer letter-spacing)
	(gimp-text-layer-set-line-spacing text-layer line-spacing)		
    (gimp-image-resize-to-layers image)	
    (resize-image-to-layers-by-border-lb image logo-layer border))      
    (script-fu-lizard-breath-efect image text-layer ScaleSize color thickness colorin ScaleShape displace-type conserve)
	
	(if (= conserve FALSE) (begin (gimp-image-merge-visible-layers image EXPAND-AS-NECESSARY)))

    (gimp-display-new image)
)
)
(script-fu-register
	"script-fu-lizard-breath-logo"
	"Lizard-breath..."
	"creates a reptilian scales effect from a text layer"
	"Rod/MrQ"
	"GPL"
	"2012/2020"
	""
	;SF-IMAGE       "The Image" 0
	;SF-DRAWABLE    "The Layer" 0
	SF-TEXT       _"Text"              "GIMP\nCHAT"
	SF-OPTION     _"Direction"                 '("From left to right"  "From right to left" "Vertical, right to left (mixed orientation)" "Vertical, right to left (upright orientation)" "Vertical, left to right (mixed orientation)" "Vertical, left to right (upright orientation)")
	SF-OPTION     _"Justify" '("Centered" "Left" "Right" "Fill")
	SF-ADJUSTMENT _"Letter Spacing" '(0 -100 100 1 5 0 0) 
	SF-ADJUSTMENT _"Line Spacing" '(0 -100 100 1 5 0 0)    
	SF-ADJUSTMENT _"Font size (pixels)"  '(100 2 1000 1 10 0 1)
	SF-ADJUSTMENT _"Add Border (empty margin)" '(20 0 1000 1 5 0 0) 
	SF-FONT       _"Font"                "RoostHeavy"	
	SF-ADJUSTMENT  _"Scale Size"            '(13 5 100 1 10 0 SF-SLIDER)
	SF-COLOR       _"Color Text Edge"       '(200 215 20)
	SF-ADJUSTMENT _"Thickness Text Edge" '(2 1 200 1 5 0 0)
	SF-COLOR       _"Color Inside the Text"     '(35 115 10)	
	SF-OPTION      _"Scale Shape"           '("Squares" "Hexagons" "Octagons and Squares" "Triangles")
	SF-OPTION      _"Displace Type"         '("WRAP" "SMEAR" "BLACK")
	SF-TOGGLE     _"Keep the Layers"   TRUE	
)
(script-fu-menu-register "script-fu-lizard-breath-logo" "<Image>/File/Create/Logos" )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (script-fu-lizard-breath-logo-alpha 
                   img                     ; Image
                   logo-layer              ; Input layer
				   create-new
				   ScaleSize
				   color
				   thickness
				   colorin
				   ScaleShape
				   displace-type
				   conserve)
				   
; star layer to new image - add MrQ ----------------------------------------------------------
				   
       (if (= create-new TRUE)
	        (begin
			    (let*(
					(active_layer (car (gimp-image-get-active-layer img)))
					(image-type (car (gimp-image-base-type img)))
					(width (car (gimp-drawable-width active_layer)))
					(height (car (gimp-drawable-height active_layer)))
					(new-image 0)
					(new-layer 0)
					(aa 0)
					)
					(set! new-image (car (gimp-image-new width height image-type)))
					(gimp-display-new new-image)
					(set! new-layer (car (gimp-layer-new-from-drawable logo-layer new-image)))
					(set! aa (gimp-image-insert-layer new-image new-layer 0 0))
					(script-fu-lizard-breath-efect new-image new-layer ScaleSize color thickness colorin ScaleShape displace-type conserve)
				    (if (= conserve FALSE)(begin (gimp-image-merge-visible-layers new-image EXPAND-AS-NECESSARY)))
				    (gimp-displays-flush)
                )
			)
		)
; end layer to new image - add MrQ ----------------------------------------------------------	
	
	 (if (= create-new FALSE)				   
         (let* (
				(begin	
					(script-fu-lizard-breath-efect img logo-layer ScaleSize color thickness colorin ScaleShape displace-type conserve)
					 (if (= conserve FALSE)(begin (gimp-image-merge-visible-layers img EXPAND-AS-NECESSARY)))
				(gimp-displays-flush)		
	            )			   
              )
	    )
    )
)
(script-fu-register
	"script-fu-lizard-breath-logo-alpha"
	"Lizard-breath..."
	"creates a reptilian scales effect from active layer"
	"Rod/MrQ"
	"GPL"
	"2012/2020"
	"*"
	SF-IMAGE       "The Image" 0
	SF-DRAWABLE    "The Layer" 0
	SF-TOGGLE     _"Active Layer to New Image"   TRUE
	SF-ADJUSTMENT  _"Scale Size"            '(13 5 100 1 10 0 SF-SLIDER)
	SF-COLOR       _"Color Text Edge"       '(200 215 20)
	SF-ADJUSTMENT _"Thickness Text Edge"    '(2 1 200 1 5 0 0)
	SF-COLOR       _"Color Inside the Text"     '(35 115 10)	
	SF-OPTION      _"Scale Shape"           '("Squares" "Hexagons" "Octagons and Squares" "Triangles")
	SF-OPTION      _"Displace Type"         '("WRAP" "SMEAR" "BLACK")
	SF-TOGGLE     _"Keep the Layers"   TRUE	
)
(script-fu-menu-register "script-fu-lizard-breath-logo-alpha" "<Image>/Filters/Alpha to Logo" )				   
				